@@ -3,7 +3,7 @@
 # @时间    :2022/12/1 23:45
 # @作者    :Zed
 # @目的
-from matrix_last import Series, DataFrame, Matrix
+from matrix import Series, DataFrame, Matrix
 
 
 
