@@ -1,82 +1,11 @@
 # -*- coding:utf-8 -*-
 # Auther: Zed
-"""
-It's beautiful right? ^_^
 
-dm = Matrix()
-print('data_matrix:', dm)
-res = dm[dm['code'] == '000001']
-print('data_matrix_slice:\n', res)
-------------------------------
-data_matrix:
-             ['close', 'high', 'low', 'open', 'ret']
-100
-000001  [0.80676805 0.30489957...0.83385824 0.56749199]
-000002  [0.0158474  0.65408549...0.14293321 0.76696439]
-                  ......
-
-200
-000001  [0.62020139 0.96969446...0.76984185 0.14326731]
-000002  [0.82182821 0.10924971...0.9967727  0.51963606]
-                  ......
-                  ......
-400
-...
-000003  [0.15914842 0.99641071...0.15914842 0.99641071]
-Dimension: ['timestamp', 'code', 'features'], Shape: (4, 3, 5), dtype: matrix
-
-data_matrix_slice:
-         close      high       low      open       ret
-100  0.806768  0.304900  0.210918  0.833858  0.567492
-200  0.620201  0.969694  0.754948  0.769842  0.143267
-300  0.008385  0.087556  0.994873  0.706558  0.752703
-400  0.388965  0.946414  0.967339  0.567995  0.071485
-
-update:
-    2022-12-02 23:52  :
-        Increased function matrix.T(axis_one,axis_two), then you can transform two axis,just like pd.DataFrame().T
-        dm.T(0, 2)
-        print('-' * 30)
-        print('transform!')
-        print(dm)
-
-        dm.T('code', 'timestamp')
-        print('-' * 30)
-        print('transform!')
-        print(dm)
-    2022-12-03 0:31  :
-        Increased params type in function matrix.T, then you can use 'list' as params
-        dm.T(['code','timestamp','features'])
-        print('-' * 30)
-        print('transform!')
-        print(dm)
-    2022-12-04 22:49  :
-        Increased handle df_dict to np.ndarray and axis_name, then user can import df list easily to matrix
-        e.g:
-            data_list = os.listdir()
-            data_list = [data.split('.')[0] for data in data_list if data.endswith('.csv')]
-
-            df_dict = {}
-            for data in data_list:
-                df = pd.read_csv(data + '.csv')
-                df.index = df["trade_date"]  # 请确保dataframe.index是数据的时间,或者是有意义且不重复的值
-                df_dict[data] = df
-            dm = Matrix(df_dict, axis_name=['code', 'datetime', 'features'])
-    next plan:
-            1. Update dm element: dm.append(matrix0/DataFrame) or matrix2=em.concat([matrix0,matrix1])
-            2. IO stream: dm.read/to pandas_df,pkl,csv
-            3. Increase logic operation such as '>= <= > < !=/~ and & or / in' : dm[20220105<=datetime and code in ['000001','000002']]
-            4. Increase more __magic_function__ such as __del__: del dm["feature=='close'"] / dm.delete('datetime<=20220105')
-            5. Query function: dm.query("20220105<=datetime<=20220506 and code in ['000001','000002'] and feature.capital>float(1e9)")
-            6. Increase function fill_na: dm.fill_na('forward_value/backward_value/zero_value/avg_value')
-            7. Del dm[dm[::3]:ClassMatrix]
-            -1. More simular pd.DataFrame operation function in Matrix
-"""
-
-import numpy as np
 from typing import Union, NewType
 from copy import deepcopy
+import numpy as np
 import pandas as pd
+import pickle
 
 set_print_limit = False  # 显示有限的输出设置
 _SeriesType = Union[dict, tuple, list]
@@ -93,7 +22,7 @@ def row_fill_na(data: pd.DataFrame, fill_list) -> pd.DataFrame:
     _index.extend(fill_list)
     new_df = pd.DataFrame(new_data, index=_index, columns=data.columns)
     new_df.sort_index(inplace=True)
-    # new_df = pd.concat([data, new_df])
+
     return new_df
 
 
@@ -129,7 +58,7 @@ def df_dict2matrix(df_dict, axis_name=None):
             'row': list(value.index.values),
             'columns': list(value.columns.values),
         }
-    # def value_fill_na_check():
+
     values_dict = {}
     for key, value in deepcopy(df_info_dict).items():
         diff_row = diff_calcu(row, value['row'])
@@ -161,10 +90,8 @@ def df_dict2matrix(df_dict, axis_name=None):
 
     axis_name = {axis_name[0]: index, axis_name[1]: row, axis_name[2]: columns}
     _array = np.empty([1, len(row), len(columns)])
-    # print(_array)
 
     for key, value in values_dict.items():
-        # print(value)
         _array = np.vstack((_array, [value.values]))
     _array = _array[1:]
     return _array, axis_name
@@ -219,6 +146,15 @@ def check_Series_dtype(dtype):
     return dtype
 
 
+def check_Matrix_axis_length(matrix_data: np.ndarray, matrix_axis_name: dict):
+    _shape = matrix_data.shape
+    i = 0
+    for key, value in matrix_axis_name.items():
+        if len(value) != _shape[i]:
+            raise ValueError(f'Error length! The axis_name length {_shape[i]}!= data length {len(value)}')
+        i += 1
+
+
 # axis_name = ['timestamp', 'code', 'features']
 def _print(data: _SeriesType, head=0, tail=0):
     _print_len = 10
@@ -318,6 +254,9 @@ class Series:
         else:
             self.dtype = check_Series_dtype(_type)
 
+    def sort(self):
+        pass
+
     @property
     def values(self):
         return self._series_value
@@ -344,6 +283,8 @@ class Series:
 
             return Series(_res, self.name)
 
+
+
     def __str__(self):
         _data = _print(self._series)
         info = f'Name: {self.name}, Length:{self.length}, dtype:{self.dtype}'
@@ -353,8 +294,6 @@ class Series:
         return str(self)
 
 
-
-
 _MatrixGetSingleItemType = Union[str, int, Series]
 _MatrixGetMultiItemType = Union[str, int, Series]
 _MatrixAxisNameType = Union[dict, list]
@@ -393,6 +332,10 @@ class DataFrame:
         return _df
 
 
+_MatrixAppendObjType = Union[np.ndarray, pd.DataFrame]
+_MatrixAppendAxisName = Union[str, int]
+
+
 class Matrix:
     # 可接收多个pd.DataFrame聚合为Matrix数据类型
     # 标准样例：第一维度为时间，第二维度为合约标的代码，第三维度为特征因子features
@@ -418,15 +361,21 @@ class Matrix:
 
         # list, tuple, dict 内部装着pd.dataframe
 
-        elif type(data) != np.ndarray or not axis_name:
+        elif type(data) != np.ndarray and not axis_name:
             # test data:
             axis_name = {'timestamp': [100, 200, 300, 400], 'code': ['000001', '000002', '000003'],
                          'features': ['close', 'high', 'low', 'open', 'ret']}
             data = np.random.random([4, 3, 5])
-            # big data
-            # axis_name = {'timestamp': list(range(1000)), 'code': list(range(5000)),
-            #              'features': list(range(100))}
-            # data = np.random.random([1000, 5000, 100])
+
+        check_Matrix_axis_length(data, axis_name)
+        # elif demo:
+        # axis_name = {'timestamp': [100, 200, 300, 400], 'code': ['000001', '000002', '000003'],
+        #              'features': ['close', 'high', 'low', 'open', 'ret']}
+        # data = np.random.random([4, 3, 5])
+        # big data
+        # axis_name = {'timestamp': list(range(500, 1500)), 'code': ['00000' + str(i) for i in range(2,5000)],
+        #              'features': ['close', 'high', 'low', 'open',  'ret']}
+        # data = np.random.random([1000, 4998, 5])
 
         self.data = data
         self.axis_name = axis_name
@@ -440,14 +389,10 @@ class Matrix:
 
         self._axis_name_dict = {}
         self._axis_name_key = []
+        self._axis_value_dtype = {}
+        self._axis_value_num_dtype = {}
         self.init()
 
-    # 转置的概念：要借助numpy进行0，1，2轴任意转置
-    def pandas_df_to_matrix(self, df_dict: dict):
-        # 把多个pandas.dataframe数据放入df_dict中，然后循环处理为numpy.ndarray矩阵，最终变为matrix类
-
-        pass
-
     def T(self, axis_one, axis_two=None):
         if type(axis_one) == int and type(axis_two) == int:
             axis_one_num, axis_two_num = axis_one, axis_two
@@ -503,17 +448,90 @@ class Matrix:
         self.axis_name = _axis_name_dict
         self.init()
 
+    def append(self, obj: _MatrixAppendObjType, name=None, axis_name=None):
+        # axis_name:如果用户能输入轴名，那么就不需要作维度相等判断，因此性能会更高
+        # 添加轴的轴value如datetime的value20220501
+        # dm.append(pd.DataFrame,20220501,axis_name='datetime')
+        if type(axis_name) == str:
+            if axis_name in self._axis_name_key:
 
-    def append(self,auto_fix=False):
-        # auto_fix=True:如果用户能确保加入的维度和Matrix.value维度一样，那么就不需要作维度相等判断，因此性能会更高
-        pass
+                _axis_dimension = self._dimension_dict[axis_name]
+            else:
+                raise ValueError(f'Not exist {axis_name} in {self._axis_name_key}')
+        elif type(axis_name) == int and axis_name in [0, 1, 2]:
+            if axis_name in [0, 1, 2]:
+                _axis_dimension = axis_name
+            else:
+                raise ValueError(f'Not exist {axis_name} dimension in [0, 1, 2]')
+        else:
+            if not axis_name:
+                _axis_dimension = axis_name = 0
+            else:
+                raise ValueError(f'Matrix not exist the axis_name: {axis_name}')
 
-    def concat(self):
+        if type(name) != self._axis_value_num_dtype[_axis_dimension]:
+            raise TypeError(
+                f"The type of {_axis_dimension} dimension's axis_value '{name}' is error, you should input type {self._axis_value_num_dtype[_axis_dimension]}")
+
+        if name in self._axis_name_dict[self._dimension_num_dict[_axis_dimension]]:
+            raise ValueError(f'Existed name {name} in axis')
+        if _axis_dimension == 0:
+            remain_axis_dimension = [1, 2]
+        elif _axis_dimension == 1:
+            remain_axis_dimension = [0, 2]
+        else:
+            remain_axis_dimension = [0, 1]
+        if type(obj) == np.ndarray:
+            obj = obj
+        elif type(obj) == pd.DataFrame:
+            obj = obj.values
+
+        if obj.shape[0] == self.data.shape[remain_axis_dimension[0]] and obj.shape[1] == self.data.shape[
+            remain_axis_dimension[1]]:
+            obj = obj
+
+        elif obj.shape[0] == self.data.shape[remain_axis_dimension[1]] and obj.shape[1] == self.data.shape[
+            remain_axis_dimension[0]]:
+            obj = obj.T
+
+        else:
+            raise ValueError(
+                f'Error df_obj.shape:{obj.shape}!={remain_axis_dimension} or [{remain_axis_dimension[1], remain_axis_dimension[0]}]')
+        if _axis_dimension == 0:
+            obj = obj.reshape([1, obj.shape[0], obj.shape[1]])
+        elif _axis_dimension == 1:
+            obj = obj.reshape([obj.shape[0], 1, obj.shape[1]])
+        else:
+            obj = obj.reshape([obj.shape[0], obj.shape[1], 1])
+        new_matrix = np.append(self.data, obj, axis=_axis_dimension)
+
+        self.data = new_matrix
+        _dimension_name = self._dimension_num_dict[_axis_dimension]
+
+        self._axis_name_dict[_dimension_name].update({name: self.data.shape[_axis_dimension] - 1})
+        self.axis_name[_dimension_name].append(name)
+
+    def concat(self, obj_matrix):
         # 拼接两个matrix数据
-        pass
+        # 自动确定相同的axis_name,第三条axis轴自然确定
+
+        return concat(self, obj_matrix)
+
+    def axis_value_type_check(self):
+        # 检查下axis各轴值的类型
+        # self._axis_value_dtype = {}
+        i = 0
+        for key, values in self.axis_name.items():
+            self._axis_value_dtype[key] = type(values[0])
+            self._axis_value_num_dtype[i] = type(values[0])
+            i += 1
+            for value in values[1:]:
+                if type(value) != self._axis_value_dtype[key]:
+                    raise TypeError(
+                        f'The axis {key} type is not unified ({type(value)}!={self._axis_value_dtype[key]}')
 
     def init(self):
-
+        self.axis_value_type_check()
         if len(self.data.shape) != 3:
             raise ValueError(f'error data shape {self.data}')
         if len(self.axis_name.keys()) != 3:
@@ -554,6 +572,59 @@ class Matrix:
         #
         pass
 
+    def sort(self, sort_axis_name: Union[str, int], assign_list: list = None, reverse=False):
+        # 根据axis_name对应的轴值，进行二维矩阵的排序,可以根据指定的顺序列表assign_list（代替轴值）进行排序，
+        # 可配合Series和features进行元素值排序，如根据某一天datetime的features.close的大小进行排序
+        # 排序完后，dm.axis_name和dm._axis_name_dict需要重新赋值
+        # reverse=False表示升序
+        if type(sort_axis_name) == int:
+            if sort_axis_name not in [0, 1, 2]:
+                raise ValueError(f'Not exist {sort_axis_name} in [0,1,2]')
+            _axis_value = self.axis_name[self._dimension_num_dict[sort_axis_name]]
+            _dimension = sort_axis_name
+            _name = self._dimension_num_dict[sort_axis_name]
+        elif type(sort_axis_name) == str:
+            if sort_axis_name not in self._axis_name_key:
+                raise ValueError(f'Not exist {sort_axis_name} in {self._axis_name_key}')
+            _axis_value = self.axis_name[sort_axis_name]
+            _dimension = self._dimension_dict[sort_axis_name]
+            _name = sort_axis_name
+        else:
+            raise TypeError(f'Error type sort_axis_name: {sort_axis_name}')
+
+        if assign_list:
+            _axis_value = assign_list
+        else:
+            _axis_value.sort(reverse=reverse)
+
+        sort_index = [self._axis_name_dict[_name][sort_value] for sort_value in _axis_value]
+
+        if _dimension == 0:
+            self.data = self.data[sort_index, :, :]
+        elif _dimension == 1:
+            self.data = self.data[:, sort_index, :]
+        else:
+            self.data = self.data[:, :, sort_index]
+        self.axis_name[_name] = _axis_value
+        self._axis_name_dict[_name] = dict(zip(_axis_value, list(range(len(_axis_value)))))
+
+    def sort_value(self, ):
+        # dm.sort_value(dm[['datetime'==20220501,'futures'=='close']])
+        # 根据两个维度确定的一条Series的值大小进行排序，比如datetime=20220501，futures=close，则确定20220501当天close从大到小进行排序
+        pass
+
+    def sort_index(self, reverse=False):
+        # 根据轴值排序
+        pass
+
+    def sort_row(self, reverse=False):
+        # 根据轴值排序
+        pass
+
+    def sort_columns(self, reverse=False):
+        # 根据轴值排序
+        pass
+
     def __getitem__(self, *item):
 
         if len(item) == 1:
@@ -679,6 +750,13 @@ class Matrix:
             print('_MatrixGetMultiItemType')
             pass
 
+    def to_pkl(self, save_name: str):
+        to_pkl(self, save_name)
+
+    def to_df(self, is_matrix_index=True, matrix_index_split='.'):
+        _df = to_pandas(self, is_matrix_index=is_matrix_index, matrix_index_split=matrix_index_split)
+        return _df
+
     @property
     def values(self):
         return self.data
@@ -698,7 +776,7 @@ class Matrix:
                 last_row = list(self._axis_name_dict[_reverse_dimension[1]].keys())[-1]
                 last_data = self.data[-1][-1]
                 if len(last_data) >= 5:
-                    last_data = str(last_data[:2])[:-1] + '...' + str(last_data[:2])[1:]
+                    last_data = str(last_data[:2])[:-1] + '...' + str(last_data[-2:])[1:]
 
                 _print_data += '                  ......\n' + str(last_count) + '\n' + '...\n' + str(
                     last_row) + '  ' + str(last_data)
@@ -726,4 +804,226 @@ class Matrix:
         return self._print()
 
 
+_FillNaType = Union[str, int]
+
+
+def diff_axis_check(matrix_obj: Matrix, axis_name: str = None):
+    # 知道其中一个或者两个的轴名，返回剩下轴名
+    return list(set(matrix_obj._axis_name_key) - {axis_name})
+
+
+def matrix_fill_na(matrix_obj: Matrix, axis_name: _FillNaType = None, name='UnDefinedName'):
+    # 三维矩阵在某一维度上增加一块na二维矩阵，用于在concat前进行统一另外两个维度
+    # name：添加矩阵元素的键名
+    # axis_name：指定轴名添加na矩阵
+    if type(axis_name) == str:
+        if axis_name in matrix_obj._axis_name_key:
+
+            _dimension = matrix_obj._dimension_dict[axis_name]
+        else:
+            raise TypeError(f'Error type of {axis_name}')
+
+    elif type(axis_name) == int:
+        # int 为轴维度
+        if axis_name in [0, 1, 2]:
+            _dimension = axis_name
+
+        else:
+            raise ValueError(f'Error value of {axis_name}')
+
+    else:
+        raise TypeError(f'Error type of {axis_name}')
+
+    _shape = matrix_obj.values.shape
+    if _dimension == 0:
+        _na_matrix_shape = [_shape[1], _shape[2]]
+    elif _dimension == 1:
+        _na_matrix_shape = [_shape[0], _shape[2]]
+    else:
+        _na_matrix_shape = [_shape[0], _shape[1]]
+    _na_matrix = np.full(_na_matrix_shape, np.nan)
+
+    matrix_obj.append(_na_matrix, name, axis_name=_dimension)
+
+    return matrix_obj
+
+
+_ConcatFillNaType = Union[str, list]
+
+
+def _matrix_fill_na(matrix0, matrix1, fill_na_axis):
+    for _axis_name in fill_na_axis:
+        if _axis_name == 'index':
+            _axis_value = 0
+            _matrix0_axis_value = matrix0.index
+            _matrix1_axis_value = matrix1.index
+
+        elif _axis_name == 'row':
+            _axis_value = 1
+            _matrix0_axis_value = matrix0.row
+            _matrix1_axis_value = matrix1.row
+        else:
+            _axis_value = 2
+            _matrix0_axis_value = matrix0.columns
+            _matrix1_axis_value = matrix1.columns
+
+        matrix1_diff_value = [_matrix0 for _matrix0 in _matrix0_axis_value if
+                              _matrix0 not in _matrix1_axis_value]
+        matrix0_diff_value = [_matrix1 for _matrix1 in _matrix1_axis_value if
+                              _matrix1 not in _matrix0_axis_value]
+
+        if matrix0_diff_value:
+
+            for _diff_value in matrix0_diff_value:
+                matrix0 = matrix_fill_na(matrix0, _axis_value, _diff_value)
+
+            matrix0.sort(sort_axis_name=_axis_value)
+        if matrix1_diff_value:
+            for _diff_value in matrix1_diff_value:
+                matrix1 = matrix_fill_na(matrix1, _axis_value, _diff_value)
+
+            matrix1.sort(sort_axis_name=_axis_value)
+    # ToDo 还差一步：把补充的数据进行排序
+    # matrix0.sort(axis_name=_axis_value),matrix1.sort(axis_name=_axis_value)
+
+    return matrix0, matrix1
+
+
+def concat(matrix0: Matrix, matrix1: Matrix, fill_na_axis: _ConcatFillNaType = None) -> Matrix:
+    # 拼接两个matrix数据
+    # 自动确定相同的axis_name以及长度,第三条axis轴自然确定
+    # 因为是自动拼接，因此数据要求其中两个维度等长，第三个元素完全不相等
+    # 当要求等长的两个维度其中一个不等或者两个都不等时：
+    # fill_na_axis:指定自动填充na的维度index，row，columns,使该维度轴值完全相同
+    """
+    demo:
+
+    new_dm = concat(dm0, dm1, ['row','columns'])
+
+    new_dm = concat(dm0, dm1, 'row')
+
+    """
+    if fill_na_axis:
+        if type(fill_na_axis) == str:
+            if fill_na_axis not in ['index', 'row', 'columns']:
+                raise ValueError(f'Error value fill_na_axis: {fill_na_axis}')
+            matrix0, matrix1 = _matrix_fill_na(matrix0, matrix1, [fill_na_axis])
+
+
+        elif type(fill_na_axis) == list:
+            if fill_na_axis not in [['index', 'row', 'columns'],
+                                    ['row', 'columns'],
+                                    ['index', 'columns'],
+                                    ['index', 'row'],
+                                    ]:
+                raise ValueError(f'Error value fill_na_axis: {fill_na_axis}')
+            matrix0, matrix1 = _matrix_fill_na(matrix0, matrix1, fill_na_axis)
+
+
+        else:
+            raise TypeError(f'Error type of fill_na_axis: {fill_na_axis}')
+    # print('res__________________')
+    # print('data',matrix0.values, matrix1.values)
+    matrix0_axis_name_dict = matrix0._axis_name_dict
+    matrix0_axis_name_keys = list(matrix0_axis_name_dict.keys())
+
+    matrix0_axis_name_values = list(matrix0.axis_name.values())
+    matrix1_axis_name_dict = matrix1._axis_name_dict
+    matrix1_axis_name_keys = list(matrix1_axis_name_dict.keys())
+
+    matrix1_axis_name_values = list(matrix1.axis_name.values())
+
+    same_axis_value = []
+    same_axis_keys = []
+    diff_axis_name = ''
+    diff_axis_value = {}
+
+    for i in range(len(matrix0_axis_name_values)):
+
+        if matrix0_axis_name_values[i] == matrix1_axis_name_values[i]:
+            same_axis_value.append(matrix0_axis_name_values[i])
+            same_axis_keys.append(matrix0_axis_name_keys[i])
+        else:
+            diff_axis_name = matrix0_axis_name_keys[i]
+            diff_axis_value[0] = matrix0_axis_name_values[i]
+            diff_axis_value[1] = matrix1_axis_name_values[i]
+
+    if matrix0_axis_name_keys != matrix1_axis_name_keys:
+        raise ValueError(f"matrix0's {matrix0_axis_name_keys} != matrix1's {matrix0_axis_name_keys}")
+
+    if len(same_axis_value) != 2:
+        raise ValueError(
+            f"matrix0 and matrix1 same_axis_value must be 2 ,but you have {len(same_axis_value)} (diff axis: {diff_axis_value})")
+
+    if list(set(diff_axis_value[0]) & set(diff_axis_value[1])):
+        raise ValueError(
+            f'Same axis_value matrix0 and axis_matrix1 in {diff_axis_name}({diff_axis_value[0]}'
+            f' and {diff_axis_value[1]}) you should keep different')
+
+    for i in range(len(matrix0_axis_name_keys)):
+        if matrix0_axis_name_dict[matrix0_axis_name_keys[i]] in same_axis_value:
+            same_axis_keys.append(matrix0_axis_name_keys[i])
+
+    if matrix0._dimension_dict[diff_axis_name] != matrix1._dimension_dict[diff_axis_name]:
+        raise ValueError(f"Matrix0's {diff_axis_name} dimension != matrix1's {diff_axis_name} dimension")
+    if matrix0._dimension_dict[same_axis_keys[0]] != matrix1._dimension_dict[same_axis_keys[0]]:
+        raise ValueError(f"Matrix0's {same_axis_keys[0]} dimension != matrix1's {same_axis_keys[0]} dimension")
+    if matrix0._dimension_dict[same_axis_keys[1]] != matrix1._dimension_dict[same_axis_keys[1]]:
+        raise ValueError(f"Matrix0's {same_axis_keys[1]} dimension != matrix1's {same_axis_keys[1]} dimension")
+
+    new_matrix_value = np.concatenate((matrix0.values, matrix1.values), axis=matrix0._dimension_dict[diff_axis_name])
+
+    new_diff_axis = matrix0.axis_name[diff_axis_name] + matrix1.axis_name[diff_axis_name]
+    _axis_name = {}
+    for i in range(len(matrix0_axis_name_keys)):
+        if matrix0_axis_name_keys[i] == diff_axis_name:
+            _axis_name.update({diff_axis_name: new_diff_axis})
+        else:
+            _axis_name.update({matrix0_axis_name_keys[i]: matrix0.axis_name[matrix0_axis_name_keys[i]]})
+
+    return Matrix(new_matrix_value, _axis_name, True)
+
+
+def to_pkl(obj: Matrix, dir_name):
+    with open(dir_name, 'wb') as f:  # 打开文件
+        pickle.dump((obj.data, obj.axis_name, obj.unique), f)  # 用 dump 函数将 Python 对象转成二进制对象文件
+
+
+def read_pkl(dir_name):
+    with open(dir_name, 'rb') as f:  # 打开文件
+        _obj = pickle.load(f)  # 用 dump 函数将 Python 对象转成二进制对象文件
+    return Matrix(_obj[0], _obj[1], _obj[2])
+
+
+def read_pandas(obj: pd.DataFrame, shape=None, axis_name: dict = None):
+    dm_value = obj.values.reshape(shape)
+    dm = Matrix(dm_value, axis_name)
+    return dm
+
+
+def to_pandas(obj: Matrix, is_matrix_index: bool = True, matrix_index_split: str = '.') -> pd.DataFrame:
+    new_dm_data = obj.data.reshape([obj.data.shape[0] * obj.data.shape[1], obj.data.shape[2]])
+    _index = obj.index
+    _row = obj.row
+    _columns = obj.columns
+
+    _index_same_list = []
+    _index_list = []
+    for i in range(len(_index)):
+        _index_same_list.extend([_index[i] for j in range(len(_row))])
+    row_list = _row * len(_index)
+    new_array = np.append(np.array([_index_same_list]), np.array([row_list]), axis=0).T
+    new_array = np.append(new_array, np.array(new_dm_data), axis=1)
+
+    if is_matrix_index:
+        index = [str(_index_same_list[i]) + matrix_index_split + str(row_list[i]) for i in range(len(_index_same_list))]
+    else:
+        index = list(range(new_array.shape[0]))
+    columns = [obj.index_name, obj.row_name]
+    columns.extend(obj.columns)
+    df = pd.DataFrame(new_array, index=index, columns=columns)
+
+    return df
+
+
 
